public class IntSearchTree {
    private IntTreeNode overallRoot;

    // Post: Constructs an empty tree
    public IntSearchTree() {
        this.overallRoot = null;
    }

    // Pre: values is sorted
    // Post: Constructs a balanced binary search tree containing the values
    public IntSearchTree(int[] values) {
        this.overallRoot = buildBST(values, 0, values.length-1);
    }

    // Pre: values is sorted
    // Post: Returns a BST built on the values between indices 
    //       start and end inclusive.
    private IntTreeNode buildBST(int[] values, int start, int end) {
        if (start > end) {
            return null;
        }
        if (start == end) {
            return new IntTreeNode(values[start]);
        }
        int mid = start + (end - start) / 2;
        IntTreeNode node = new IntTreeNode(values[mid]);
        node.left = buildBST(values, start, mid-1);
        node.right = buildBST(values, mid+1, end);
        return node;
    }

    // Post: Returns whether or not the provided value is present within this search tree
    public boolean contains(int value) {
        // TODO: Implement!
        return false;
    }

    // Post: Adds the provided value to this search tree
    public void add(int value) {
        // TODO: Implement!
    }

    // Post: Returns a string representation of the current tree, rotated 90-degrees
    //       counter-clockwise
    @Override
    public String toString() {
        return toString(overallRoot, "");
    }

    // Post: Returns a string representation of the given subtree at the provided
    //       indent level, rotated 90-degrees counter-clockwise
    private String toString(IntTreeNode root, String indent) {
        if (root != null) {
            return toString(root.right, indent + "   ") +
                   indent + root.data + "\n" +
                   toString(root.left, indent + "   ");
        }
        return "";
    }   

    // Class that represents a single node in the tree
    private static class IntTreeNode {
        public int data;    // data stored at this node
        public IntTreeNode left;    // reference to left subtree
        public IntTreeNode right;   // reference to right subtree

        // Post: Constructs a leaf node with the given data.
        public IntTreeNode(int data) {
            this(data, null, null);
        }

        // Post: Constructs a leaf or branch node with the given data and links.
        public IntTreeNode(int data, IntTreeNode left, IntTreeNode right) {
            this.data = data;
            this.left = left;
            this.right = right;
        }
    }
}
