// This class represents a tree of integers
public class IntTree {
    private IntTreeNode overallRoot;

    public IntTree(int[] values) {
        this.overallRoot = buildTree(values, 0);
    }

    private IntTreeNode buildTree(int[] values, int index) {
        if (index >= values.length) {
            return null;
        } else {
            IntTreeNode node = new IntTreeNode(values[index]);
            node.left = buildTree(values, 2 * index + 1);
            node.right = buildTree(values, 2 * index + 2);
            return node;
        }
    }

    // Post: Returns whether or not the provided value is present within the tree
    public boolean contains(int value) {
        return contains(value, overallRoot);
    }

    // Post: Returns whether or not the provided value is present within the subtree
    //       rooted at 'currentRoot'
    private boolean contains(int value, IntTreeNode currentRoot) {
        if (currentRoot == null) {
            return false;
        } else {
            return currentRoot.data == value ||
                   contains(value, currentRoot.left) ||
                   contains(value, currentRoot.right);
        }
    }

    // Post: Returns a string representation of the current tree, rotated 90-degrees
    //       counter-clockwise
    @Override
    public String toString() {
        return toString(overallRoot, "");
    }

    // Post: Returns a string representation of the given subtree at the provided
    //       indent level, rotated 90-degrees counter-clockwise
    private String toString(IntTreeNode root, String indent) {
        if (root != null) {
            return toString(root.right, indent + "   ") +
                   indent + root.data + "\n" +
                   toString(root.left, indent + "   ");
        }
        return "";
    }  

    // Class that represents a single node in the tree
    private static class IntTreeNode {
        public int data; // data stored at this node
        public IntTreeNode left; // reference to left subtree
        public IntTreeNode right; // reference to right subtree

        // Constructs a leaf node with the given data.
        public IntTreeNode(int data) {
            this(data, null, null);
        }

        // Constructs a leaf or branch node with the given data and links.
        public IntTreeNode(int data, IntTreeNode left, IntTreeNode right) {
            this.data = data;
            this.left = left;
            this.right = right;
        }
    }
}
